/**
 * Poynt — a GoDaddy Brand for WooCommerce.
 *
 * Payment form handler script.
 *
 * @since 1.0.0
 */
jQuery( ( $ ) => {

	'use strict';

	/**
	 * Payment form handler.
	 *
	 * Interacts with the Poynt Collect API to process a checkout payment form.
	 *
	 * @link https://docs.poynt.com/app-integration/poynt-collect/#poynt-collect
	 *
	 * @since 1.0.0
	 */
	window.WC_Poynt_Payment_Form_Handler = class WC_Poynt_Payment_Form_Handler extends SV_WC_Payment_Form_Handler_v5_10_3 {

		/**
		 * Instantiates the payment form handler.
		 *
		 * Loads the payment handler and intercepts form submissions to inject the token returned by Poynt Collect API.
		 *
		 * @since 1.0.0
		 *
		 * @param {Object} args form handler arguments
		 */
		constructor( args ) {

			super( args );

			this.appId            = args.appId;
			this.businessId       = args.businessId;
			this.customerName     = args.customerName;
			this.customerAddress  = args.customerAddress;
			this.isLoggingEnabled = args.isLoggingEnabled;

			// initialization on form load, for non-checkout pages
			if ( ! this.formInitialized && ! $( 'form.checkout' ).length ) {
				this.initForm();
			}

			$( document.body ).on( 'sv_wc_payment_form_valid_payment_data', ( event, data ) => {

				if ( data.payment_form && data.payment_form.id !== this.id ) {
					return data.passed_validation;
				}

				// if regular validation passed
				if ( data.passed_validation ) {

					// we have a token if there's a nonce or a saved method is selected
					if ( this.saved_payment_method_selected || this.hasNonce() ) {
						return $( document.body ).triggerHandler( 'wc_poynt_form_submitted', { payment_form: data.payment_form } ) !== false;
					}

					// override the loaded address data if available via form fields
					let customerAddress   = $( '#billing_address_1' ).val(),
					    customerPostcode  = $( '#billing_postcode' ).val(),
					    customerFirstName = $( '#billing_first_name' ).val(),
					    customerLastName  = $( '#billing_last_name' ).val();

					if ( customerAddress && customerAddress.length > 0 ) {
						this.customerAddress.line1 = customerAddress;
					}
					if ( customerPostcode && customerPostcode.length > 0 ) {
						this.customerAddress.postcode = customerPostcode;
					}
					if ( customerFirstName && customerFirstName.length > 0 ) {
						this.customerName.firstName = customerFirstName;
					}
					if ( customerLastName && customerLastName.length > 0 ) {
						this.customerName.lastName = customerLastName;
					}

					// block the UI
					this.form.block( { message: null, overlayCSS: { background: '#fff', opacity: 0.6 } } );

					// create the nonce
					this.createNonce();

					// always return false to resubmit the form
					return false;
				}
			} );

			// clear the payment nonce on errors
			$( document.body ).on( 'checkout_error', () => {
				this.clearNonce();
			} );
		}

		/**
		 * Gets the nonce field.
		 *
		 * Returns a jQuery object with the hidden input that holds a nonce value.
		 *
		 * @since 1.0.0
		 *
		 * @returns {Object} jQuery object
		 */
		getNonceField() {
			return $( '#wc-' + this.id_dasherized + '-nonce' );
		}

		/**
		 * Clears the payment nonce.
		 *
		 * Resets the nonce value in the hidden input.
		 *
		 * @since 1.0.0
		 */
		clearNonce() {
			this.getNonceField().val( '' );
		}

		/**
		 * Creates a nonce using Poynt Collect.
		 *
		 * Saves the nonce to a hidden input and resubmits the form.
		 *
		 * @link https://docs.poynt.com/app-integration/poynt-collect/#creating-a-nonce
		 *
		 * @since 1.0.0
		 */
		createNonce() {

			let nonceData = {
				businessId: this.businessId
			};

			if ( this.customerName.firstName ) {
				nonceData.firstName = this.customerName.firstName;
			}

			if ( this.customerName.lastName ) {
				nonceData.lastName = this.customerName.lastName;
			}

			if ( this.customerAddress.line1 ) {
				nonceData.line1 = this.customerAddress.line1;
			}

			if ( this.customerAddress.postcode ) {
				nonceData.zip = this.customerAddress.postcode;
			}

			this.debugLog( nonceData );

			/**
			 * @link https://docs.poynt.com/app-integration/poynt-collect/#collect-getnonce
			 */
			this.collect.getNonce( nonceData );
		}

		/**
		 * Determines whether a nonce exists.
		 *
		 * Checks the hidden input for a value.
		 *
		 * @since 1.0.0
		 *
		 * @returns {boolean} whether a nonce exists
		 */
		hasNonce() {
			return this.getNonceField().val().length > 0;
		}

		/**
		 * Determines whether a postcode field is present.
		 *
		 * @since 1.0.0
		 *
		 * @returns {boolean} whether the postcode field exists
		 */
		hasPostcodeField() {
			return $( '#billing_postcode' ).length > 0;
		}

		/**
		 * Handles the error event data.
		 *
		 * Logs errors to console and maybe renders them in a user-facing notice.
		 *
		 * @since 1.0.0
		 *
		 * @param {Object} event after a form error
		 */
		handleError( event ) {

			let errorMessage = '';

			// Poynt Collect API has some inconsistency about error message response data:
			if ( 'error' === event.type && event.data ) {
				if ( event.data.error && event.data.error.message && event.data.error.message.message ) {
					errorMessage = event.data.error.message.message;
				} else if ( event.data.message ) {
					errorMessage = event.data.message;
				} else if ( event.data.error ) {
					errorMessage = event.data.error;
				}
			}

			if ( 'string' === typeof errorMessage && errorMessage.length > 0 ) {

				this.debugLog( errorMessage );

				if ( errorMessage.includes( 'Request failed' ) ) {
					super.render_errors( [ this.params.generic_error ] );
				} else if ( errorMessage.includes( 'Missing details' ) ) {
					super.render_errors( [ this.params.missing_card_details ] );
				} else if ( errorMessage.includes( 'Missing field' ) ) {
					super.render_errors( [ this.params.missing_billing_fields ] );
				} else {
					super.render_errors( [ errorMessage ] );
				}

			} else {

				this.debugLog( event );
			}
		}

		/**
		 * Handles a payment form ready event.
		 *
		 * Unblocks the payment form after initialization.
		 *
		 * @since 1.0.0
		 *
		 * @param {Object} event after the form is ready
		 */
		handlePaymentFormReady( event ) {

			if ( ! event.type || 'ready' !== event.type ) {
				this.debugLog( event );
			} else {
				this.debugLog( 'Payment form ready' );
			}

			this.form.unblock();
		}

		/**
		 * Handles a nonce ready event.
		 *
		 * Sets the nonce to hidden field and submits the form.
		 *
		 * @since 1.0.0
		 *
		 * @param {Object} payload containing the nonce
		 */
		handleNonceReady( payload ) {

			if ( payload.data && payload.data.nonce ) {
				this.getNonceField().val( payload.data.nonce );
				this.debugLog( 'Nonce set' );
			} else {
				this.clearNonce();
				this.debugLog( 'Nonce value is empty' );
			}

			this.form.submit();
		}

		/**
		 * Initializes the form.
		 *
		 * Adds listeners for the ready and error events.
		 *
		 * @link https://docs.poynt.com/app-integration/poynt-collect/#collect-mount
		 *
		 * @since 1.0.0
		 */
		initForm() {

			// run only once
			if ( this.initializingForm ) {
				return;
			}

			this.initializingForm = true;

			this.collect = new TokenizeJs( this.businessId, this.appId );

			let showZip = ! this.customerAddress.postcode && ! this.hasPostcodeField();

			/**
			 * Initialize the Payment Form with Poynt Collect API.
			 *
			 * For configuration options, see:
			 * @link https://docs.poynt.com/app-integration/poynt-collect/#collect-mount
			 *
			 * For CSS options, see:
			 * @link https://docs.poynt.com/app-integration/poynt-collect/#passing-in-custom-css-optional
			 * @link https://github.com/medipass/react-payment-inputs#styles
			 */
			this.collect.mount( 'wc-' + this.id_dasherized + '-hosted-form', document, {
				displayComponents: {
					firstName:    false,
					lastName:     false,
					emailAddress: false,
					zipCode:      showZip,
					labels:       true,
					submitButton: false,
				},
				iFrame: {
					border:       '0px',
					borderRadius: '0px',
					boxShadow:    'none',
					height:       showZip ? '280px' : '230px',
					width:        'auto',
				},
				style: {
					theme: 'checkout',
				},
			} );

			// triggers when a nonce is ready
			this.collect.on( 'nonce', payload => {
				this.handleNonceReady( payload );
			} );

			// triggers when the payment form is ready
			this.collect.on( 'ready', event => {

				this.initializingForm = false;
				this.formInitialized  = true;

				this.handlePaymentFormReady( event );
			} );

			// triggers when there is a payment form error
			this.collect.on( 'error', error => {
				this.handleError( error );
			} );
		}

		/**
		 * Sets up the payment fields.
		 *
		 * Calls parent method and initializes the payment form.
		 *
		 * @since 1.0.0
		 */
		set_payment_fields() {

			super.set_payment_fields();

			if ( this.formInitialized ) {
				this.collect.unmount( 'wc-' + this.id_dasherized + '-hosted-form', document );
				this.formInitialized = false;
			}

			if ( this.businessId && this.appId && ! this.initializingForm ) {
				this.initForm();
			}
		}

		/**
		 * Validates card data.
		 *
		 * Implements and overrides parent method (bypasses validation, handled by Poynt API).
		 *
		 * @since 1.0.0
		 *
		 * @returns {boolean} always true, as the fields are hosted
		 */
		validate_card_data() {
			return true;
		}

		/**
		 * Logs an item to console if logging is enabled.
		 *
		 * @since 1.0.0
		 *
		 * @param {String|Object} logData
		 */
		debugLog( logData ) {
			if ( this.isLoggingEnabled ) {
				console.log( logData );
			}
		}

	}

	// dispatch loaded event
	$( document.body ).trigger( 'wc_poynt_payment_form_handler_loaded' );

} );
