<?php
/**
 * Poynt — a GoDaddy Brand for WooCommerce.
 *
 * @author GoDaddy
 * @copyright Copyright (c) 2021 GoDaddy Operating Company, LLC. All Rights Reserved.
 * @license GPL-2.0
 */

namespace GoDaddy\WooCommerce\Poynt\Gateways;

use GoDaddy\WooCommerce\Poynt\Helpers\ArrayHelper;
use GoDaddy\WooCommerce\Poynt\Helpers\PoyntHelper;
use GoDaddy\WooCommerce\Poynt\Helpers\WCHelper;
use GoDaddy\WooCommerce\Poynt\Plugin;
use SkyVerge\WooCommerce\PluginFramework\v5_10_3 as Framework;
use WC_Order;

defined('ABSPATH') or exit;

/**
 * Pay in Person gateway handler.
 *
 * @since x.x.x
 *
 * @method Plugin get_plugin()
 * @method PaymentForm|null get_payment_form_instance()
 */
class PayInPersonGateway extends Framework\SV_WC_Payment_Gateway_Direct
{
    /** @var string[] default shipping methods for GoDaddy Payments - Selling in Person */
    protected $defaultEnableForMethods = ['local_pickup', 'gdp_local_delivery', 'local_pickup_plus'];

    /** @var bool shipping methods validation status. */
    protected $isShippingMethodInvalid;

    /** @var string businessId. */
    protected $businessId = '';

    /** @var int terminal original price */
    const TERMINAL_ORIGINAL_PRICE = 499;

    /** @var int terminal sale price */
    const TERMINAL_SALE_PRICE = 249;

    /** @var int payments hub URL */
    const HUB_URL = 'https://payments.godaddy.com';

    /**
     * PayInPerson gateway constructor.
     *
     * @since x.x.x
     */
    public function __construct()
    {
        $plugin = poynt_for_woocommerce();

        parent::__construct(
            Plugin::PAYINPERSON_GATEWAY_ID,
            $plugin,
            [
                'method_title'       => __('GoDaddy Payments – Selling in Person', 'godaddy-payments'),
                'method_description' => esc_html__('Customers can buy online and pay in person with orders synced to your Smart Terminal.', 'godaddy-payments'),
                'payment_type'       => Plugin::PAYINPERSON_GATEWAY_ID,
                'supports' => [
                    self::FEATURE_PRODUCTS,
                    self::FEATURE_REFUNDS,
                    self::FEATURE_ADD_PAYMENT_METHOD,
                    self::FEATURE_CREDIT_CARD_CAPTURE,
                ],
                'countries' => ['US'],
                'currencies' => ['USD'],
            ]
        );

        $this->isShippingMethodInvalid = false;

        if (! $this->title) {
            $this->title = $this->get_default_title();
        }

        $this->addHooks();

        if (is_admin()) {
            $this->businessId = PoyntHelper::getBusinessId();
        }
    }

    /**
     * Setup gateway hooks.
     *
     * @since x.x.x
     */
    private function addHooks()
    {

        // add styles for gateway
        add_action('admin_enqueue_scripts', [$this, 'addGatewayStyles']);

        // Add instructions to WC order emails sent to the customers.
        add_action('woocommerce_email_before_order_table', [$this, 'instructionsEmail'], 20, 2);
    }

    /**
     * Adds gateway setup page stylesheet.
     *
     * @since x.x.x
     */
    public function addGatewayStyles()
    {
        if (! WCHelper::isAccessingSettings(Plugin::PAYINPERSON_GATEWAY_ID)) {
            return;
        }

        wp_enqueue_style(Plugin::PAYINPERSON_GATEWAY_ID, $this->get_plugin()->get_plugin_url().'/assets/css/pay-in-person.css', [], $this->get_plugin()->get_version());
    }

    /**
     * Process a transaction.
     *
     * @since x.x.x
     *
     * @param \WC_Order $order the order object
     * @return bool
     * @throws SV_WC_Plugin_Exception
     */
    protected function do_transaction($order)
    {
        return $order;
    }

    /**
     * Render custom order received text on the thank you page.
     *
     * @since x.x.x
     *
     * @param string $text order received text
     * @param \WC_Order|null $order order object
     * @return string
     */
    public function maybe_render_held_order_received_text($text, $order)
    {

        // only show if the user checkout using pay in person payment method
        if (Plugin::PAYINPERSON_GATEWAY_ID === $order->get_payment_method()) {
            $text = $this->get_option('instructions', $this->getDefaultInstructions());
        }

        return wp_kses_post(wpautop(wptexturize($text)));
    }

    /**
     * Builds the capture handler instance.
     * Assigns it to default GoDaddy Payments - Credit Card gateway to handle capture.
     *
     * @since 1.3.0
     */
    public function init_capture_handler()
    {
        $this->capture_handler = new \GoDaddy\WooCommerce\Poynt\Gateways\GDPCapture(poynt_for_woocommerce()->get_gateway(Plugin::CREDIT_CARD_GATEWAY_ID));
    }

    /**
     * Adds instructions to WC order emails sent to the customers.
     *
     * @param WC_Order $order Order object.
     * @param bool $sentToAdmin Sent to admin.
     */
    public function instructionsEmail($order, $sentToAdmin)
    {
        if (! $sentToAdmin && Plugin::PAYINPERSON_GATEWAY_ID === $order->get_payment_method()) {
            echo wp_kses_post(wpautop(wptexturize($this->get_option('instructions', $this->getDefaultInstructions()))).PHP_EOL);
        }
    }

    /**
     * Toggle between settings page and Poynt Terminal CTA based on terminal activation status.
     *
     * @see \WC_Settings_API::admin_options()
     *
     * @since x.x.x
     */
    public function admin_options()
    {
        echo '<h2>'.esc_html($this->get_method_title());
        wc_back_link(__('Return to payments', 'godaddy-payments'), admin_url('admin.php?page=wc-settings&tab=checkout'));
        echo '</h2>';
        echo wp_kses_post(wpautop($this->get_method_description()));

        if (PoyntHelper::hasPoyntSmartTerminalActivated()):?>
            <p class="pay-in-person-settings-description">
                <?php printf(
                    esc_html__('%1$sShop Smart Terminal%2$s', 'godaddy-payments'),
                    '<a href="'.esc_url($this->getSmartTerminalProductPageUrl()).'" target="_blank">',
                    ' <span class="dashicons dashicons-external"></span></a>'
                ); ?>
                &nbsp;|&nbsp;
                <?php printf(
                    esc_html__('%1$sDevices%2$s', 'godaddy-payments'),
                    '<a href="'.esc_url(add_query_arg('businessId', $this->businessId, self::HUB_URL.'in-person/devices')).'" target="_blank">',
                    ' <span class="dashicons dashicons-external"></span></a>'
                ); ?>
                &nbsp;|&nbsp;
                <?php printf(
                    esc_html__('%1$sCatalogs%2$s', 'godaddy-payments'),
                    '<a href="'.esc_url(add_query_arg('businessId', $this->businessId, self::HUB_URL.'in-person/catalog')).'" target="_blank">',
                    ' <span class="dashicons dashicons-external"></span></a>'
                ); ?>
                &nbsp;|&nbsp;
                <?php printf(
                    esc_html__('%1$sCustomize Terminal%2$s', 'godaddy-payments'),
                    '<a href="'.esc_url(add_query_arg('businessId', $this->businessId, self::HUB_URL.'in-person/customization')).'" target="_blank">',
                    ' <span class="dashicons dashicons-external"></span></a>'
                ); ?>
            </p>
            <table class="form-table">
                <?php echo $this->generate_settings_html($this->get_form_fields(), false); ?>
            </table>
            <?php $this->display_errors(); ?>
        <?php else:
            $GLOBALS['hide_save_button'] = true; ?>
            <div class="pay-in-person-settings-no-order">
                <div class="pay-in-person-settings-no-order-upper">
                    <h4> <?php echo __('Smart Terminal', 'godaddy-payments'); ?></h4>
                    <h2> <?php echo __('Dual screens for smoother selling.', 'godaddy-payments'); ?></h2>
                    <p>  <?php echo __('Our dual screens make check out a breeze. Plus, our all-in-one terminal includes a built-in payment processor, scanner, printer, security and more.', 'godaddy-payments'); ?></p>
                </div>
                <div class="pay-in-person-settings-no-order-lower">
                    <div class="pay-in-person-settings-no-order-lower-inner">
                        <div class="pay-in-person-settings-no-order-price">
                            <span class="pay-in-person-settings-no-order-price-sale">$<?php echo static::TERMINAL_SALE_PRICE; ?></span>
                            <span class="pay-in-person-settings-no-order-price-linethrough">$<?php echo static::TERMINAL_ORIGINAL_PRICE; ?></span>
                        </div>
                        <div class="pay-in-person-settings-no-order-badges">
                            <span class="pay-in-person-settings-no-order-free"> <?php echo __('Free', 'godaddy-payments'); ?></span>
                            <span class="pay-in-person-settings-no-order-shipping"> <?php echo __('2-day shipping.', 'godaddy-payments'); ?></span>
                        </div>
                        <div class="pay-in-person-settings-no-order-btn">
                            <a target="_blank" href="<?php echo esc_url($this->getSmartTerminalProductPageUrl()); ?>"> <?php echo __('Learn More', 'godaddy-payments'); ?></a>
                        </div>
                    </div>
                </div>
            </div>
        <?php endif;
    }

    /**
     * Gets the Smart Terminal product page URL.
     *
     * @return string
     * @throws Exception
     */
    protected function getSmartTerminalProductPageUrl() : string
    {
        return sprintf(
            'https://payments.godaddy.com/in-person/shop/78d12aa5-6b75-4669-b661-8501023046c9%s',
            ! empty($this->businessId) ? '?businessId='.$this->businessId : ''
        );
    }

    /**
     * Gets the Pay in Person form fields.
     *
     * @since x.x.x
     *
     * @return array
     */
    protected function get_method_form_fields() : array
    {
        // hide debug mode
        unset($this->form_fields['debug_mode']);

        return [
            'title' => [
                'title'    => esc_html__('Title', 'godaddy-payments'),
                'type'     => 'text',
                'desc_tip' => esc_html__('Payment method title that the customer will see during checkout.', 'godaddy-payments'),
                'default'  => $this->get_default_title(),
            ],
            'instructions'       => [
                'title'       => esc_html__('Order received instructions', 'godaddy-payments'),
                'type'        => 'textarea',
                'default'     => $this->getDefaultInstructions(),
                'desc_tip'    => esc_html__('Message that the customer will see on the order received page and in the processing order email after checkout.', 'godaddy-payments'),
            ],
            'enable_for_methods' => [
                'title'             => __('Enable for Shipping Methods', 'godaddy-payments'),
                'type'              => 'multiselect',
                'class'             => 'wc-enhanced-select',
                'css'               => 'width: 400px;',
                'default'           => $this->defaultEnableForMethods,
                'options'           => $this->loadShippingMethodOptions(),
                'desc_tip'          => esc_html__('Select the shipping methods that will show this payment method for the customer during checkout.', 'godaddy-payments'),
                'custom_attributes' => [
                    'data-placeholder' => __('Select Shipping Methods', 'godaddy-payments'),
                ],
            ],
        ];
    }

    /**
     * Determines whether the gateway is configured.
     *
     * @since x.x.x
     *
     * @return bool
     */
    public function is_configured() : bool
    {
        return PoyntHelper::hasPoyntSmartTerminalActivated();
    }

    /**
     * Determines if the gateway is available at checkout.
     *
     * Note: this method is called by WooCommerce, so it needs to remain snake_case.
     *
     * @return bool
     */
    public function is_available()
    {
        return
            parent::is_available()
            && $this->isChosenShippingMethodAccepted(ArrayHelper::wrap($this->get_option('enable_for_methods')))
            && $this->orderNeedsShipping();
    }

    /**
     * Validates MultiSelect Field to determine whether the user has selected any Shipping Method.
     *
     * Note: this method is called by WooCommerce, so it needs to remain snake_case.
     *
     * @return array|null
     */
    public function validate_multiselect_field($key, $value)
    {
        if ('enable_for_methods' === $key && empty($value)) {
            $this->isShippingMethodInvalid = true;
            $this->add_error(__('At least one shipping method is required to enable Selling in Person.', 'godaddy-payments'));
        }

        return $value;
    }

    /**
     * Loads all of the shipping method options for the enable_for_methods field.
     *
     * @param array $regions for which shipping methods will be loaded
     *
     * @return array
     * @throws Exception
     */
    protected function loadShippingMethodOptions(array $regions = ['US']): array
    {
        $options = [];

        $woocommerce = $this->get_plugin()->is_plugin_active('woocommerce.php') ? WC() : null;

        // Since this is expensive, we only want to do it if we're actually on the payment settings page.
        if (! $woocommerce && ! WCHelper::isAccessingSettings(Plugin::PAYINPERSON_GATEWAY_ID)) {
            return [];
        }

        foreach ($woocommerce->shipping()->load_shipping_methods() as $method) {
            $options[$method->get_method_title()][$method->id] = $this->getShippingMethodOptionsText($method);

            foreach ($this->getAvailableShippingZones($regions) as $zone) {
                foreach ($zone->get_shipping_methods() as $shipping_method_instance_id => $shipping_method_instance) {
                    if ($shipping_method_instance->id !== $method->id) {
                        continue;
                    }

                    // Translators: %1$s shipping method title, %2$s shipping method id.
                    $option_instance_title = sprintf(esc_html__('%1$s (#%2$s)', 'godaddy-payments'), $shipping_method_instance->get_title(), $shipping_method_instance_id);

                    // Translators: %1$s zone name, %2$s shipping method instance name.
                    $option_title = sprintf(esc_html__('%1$s &ndash; %2$s', 'godaddy-payments'), $zone->get_id() ? $zone->get_zone_name() : esc_html__('Other locations', 'godaddy-payments'), $option_instance_title);

                    $options[$method->get_method_title()][$shipping_method_instance->get_rate_id()] = $option_title;
                }
            }
        }

        return $options;
    }

    /**
     * Gets the method shipping options text.
     *
     * @param \WC_Shipping_Method  $method
     * @return string
     */
    protected function getShippingMethodOptionsText(\WC_Shipping_Method $method): string
    {
        return 'local_pickup_plus' === $method->id
            ? __('Local Pickup Plus method', 'godaddy-payments')
            /* translators: Placeholders: %1$s - Shipping Method name */
            : sprintf(__('Any "%1$s" method', 'godaddy-payments'), $method->get_method_title());
    }

    /**
     * Can the order be refunded via this gateway?
     *
     * @param  WC_Order $wcOrder Order object.
     * @return bool If false, the automatic refund button is hidden in the UI.
     */
    public function can_refund_order($wcOrder)
    {
        if (! $wcOrder->get_meta('_wc_poynt_credit_card_trans_id')) {
            return false;
        }

        return parent::can_refund_order($wcOrder);
    }

    /**
     * Adds the standard refund transaction data to the order.
     *
     * Overrides the refund meta_key
     *
     * @since 1.3.0
     *
     * @param \WC_Order $order the order object
     * @param SV_WC_Payment_Gateway_API_Response $response transaction response
     */
    protected function add_refund_data(WC_Order $order, $response)
    {

        // indicate the order was refunded along with the refund amount
        $this->add_order_meta($order, 'refund_amount', $order->refund->amount);

        // add refund transaction ID
        if ($response && $response->get_transaction_id()) {
            $this->add_order_meta($order, 'refund_remoteId', $response->get_transaction_id());
        }
    }

    /**
     * Return the methods shipping options text.
     *
     * @param array $availableRegions
     * @return array
     * @throws Exception
     */
    protected function getAvailableShippingZones(array $availableRegions = []): array
    {
        $zones = [];
        $data_store = \WC_Data_Store::load('shipping-zone');
        $raw_zones = $data_store->get_zones();
        $raw_zones[] = (object) ['zone_id' => 0];

        // add only zones with accepted regions
        foreach ($raw_zones as $raw_zone) {
            $zone = new \WC_Shipping_Zone($raw_zone);
            $zone_locations = is_array($zone->get_zone_locations()) ? $zone->get_zone_locations() : [];

            $locations_filtered = array_values(
                array_filter($zone_locations,
                    function ($location) use ($availableRegions) {
                        return
                            empty($availableRegions)
                            || in_array(current(explode(':', $location->code)), $availableRegions);
                    }, ARRAY_FILTER_USE_BOTH
                )
            );

            if (! empty($locations_filtered)) {
                $zones[] = $zone;
            }
        }

        return $zones;
    }

    /**
     * Check is chosen on checkout shipping method accepted by current payment gateway.
     *
     * @param array $enabledForMethods
     *
     * @return bool
     * @throws Exception
     */
    private function isChosenShippingMethodAccepted(array $enabledForMethods = []) : bool
    {
        $woocommerce = $this->get_plugin()->is_plugin_active('woocommerce.php') ? WC() : null;

        if (! empty($enabledForMethods && $woocommerce && $woocommerce->session && $woocommerce->session->get('chosen_shipping_methods'))) {
            $chosenShippingMethod = current($woocommerce->session->get('chosen_shipping_methods'));

            if (! empty($chosenShippingMethod)) {
                $chosenShippingMethodNameElems = explode(':', $chosenShippingMethod);

                $available_methods = ArrayHelper::where($enabledForMethods, function ($method) use ($chosenShippingMethodNameElems) {
                    $methodElems = explode(':', $method);

                    return count($methodElems) == 1
                        // name consist of only 1 element - then it support any shipping method with current shipping name(id)
                        ? is_array($chosenShippingMethodNameElems) && ArrayHelper::contains(explode(':', $method), $chosenShippingMethodNameElems[0])
                        : implode(':', $chosenShippingMethodNameElems) === $method;
                });

                return ! empty($available_methods);
            }
        }

        return true;
    }

    /**
     * Determines whether the cart / order needs shipping.
     *
     * @return bool
     * @throws Exception
     */
    public function orderNeedsShipping()
    {
        $woocommerce = $this->get_plugin()->is_plugin_active('woocommerce.php') ? WC() : null;

        if ($woocommerce && $woocommerce->cart && $woocommerce->cart->needs_shipping()) {
            return apply_filters('woocommerce_cart_needs_shipping', true);
        }

        if (0 < get_query_var('order-pay') && is_page(wc_get_page_id('checkout'))) {
            $orderId = absint(get_query_var('order-pay'));

            if ($order = wc_get_order($orderId)) {
                return apply_filters('woocommerce_cart_needs_shipping', ! $this->orderIsVirtual($order));
            }
        }

        return apply_filters('woocommerce_cart_needs_shipping', false);
    }

    /**
     * Determines whether a WooCommerce order is virtual.
     *
     * @param WC_Order $order
     *
     * @return bool
     */
    protected function orderIsVirtual(WC_Order $order): bool
    {
        $isVirtual = true;

        /** @var \WC_Order_Item_Product $item */
        foreach ($order->get_items() as $item) {
            $product = $item->get_product();

            // once we've found one non-virtual product we know we're done, break out of the loop
            if ($product && ! $product->is_virtual()) {
                $isVirtual = false;
                break;
            }
        }

        return $isVirtual;
    }

    /**
     * Get the default payment method title, which is configurable within the
     * admin and displayed on checkout.
     *
     * @since x.x.x
     * @return string payment method title to show on checkout
     */
    protected function get_default_title(): string
    {
        return __('Pay in Person', 'godaddy-payments');
    }

    /**
     * Get the default payment method description, which is configurable
     * within the admin and displayed on checkout.
     *
     * @since x.x.x
     * @return string payment method description to show on checkout
     */
    protected function get_default_description(): string
    {
        return __('Pay for your order in-person at pickup or delivery.', 'godaddy-payments');
    }

    /**
     * Gets the default instructions.
     *
     * This is used to get default instructions for the Thank you order page and email.
     *
     * @since x.x.x
     *
     * @return string
     */
    private function getDefaultInstructions(): string
    {
        return esc_html__('We accept major credit/debit cards and cash.', 'godaddy-payments');
    }
}
