<?php
/**
 * Poynt — a GoDaddy Brand for WooCommerce.
 *
 * @author GoDaddy
 * @copyright Copyright (c) 2021-2022 GoDaddy Operating Company, LLC. All Rights Reserved.
 * @license GPL-2.0
 */

namespace GoDaddy\WooCommerce\Poynt;

use GoDaddy\WooCommerce\Poynt\Emails\Emails as GdpEmails;
use GoDaddy\WooCommerce\Poynt\Frontend\Admin\Notices;
use GoDaddy\WooCommerce\Poynt\Frontend\MyAccount\PaymentMethods;
use GoDaddy\WooCommerce\Poynt\Gateways\CreditCardGateway;
use GoDaddy\WooCommerce\Poynt\Gateways\PayInPersonGateway;
use GoDaddy\WooCommerce\Poynt\Helpers\PoyntHelper;
use GoDaddy\WooCommerce\Poynt\Helpers\WCHelper;
use GoDaddy\WooCommerce\Poynt\Pages\ViewOrderPage;
use GoDaddy\WooCommerce\Poynt\Payments\Captures;
use GoDaddy\WooCommerce\Poynt\Shipping\CoreShippingMethods;
use GoDaddy\WooCommerce\Poynt\Shipping\LocalPickup\LocalPickup;
use GoDaddy\WooCommerce\Poynt\Sync\Jobs\ActiveSmartTerminalDetector;
use GoDaddy\WooCommerce\Poynt\Sync\Jobs\PoyntTransactionSynchronizer;
use GoDaddy\WooCommerce\Poynt\Sync\PoyntOrderSynchronizer;
use GoDaddy\WooCommerce\Poynt\Webhooks\PoyntWebhooksHandler;
use SkyVerge\WooCommerce\PluginFramework\v5_10_3 as Framework;
use SkyVerge\WooCommerce\PluginFramework\v5_10_3\SV_WC_Payment_Gateway;

defined('ABSPATH') or exit;

/**
 * Plugin main class.
 *
 * @since 1.0.0
 *
 * @method Lifecycle get_lifecycle_handler
 */
class Plugin extends Framework\SV_WC_Payment_Gateway_Plugin
{
    /** the plugin's version number */
    const VERSION = '1.3.1';

    /** the plugin's gateway id */
    const PLUGIN_ID = 'poynt';

    /** the plugin's production environment identifier */
    const ENVIRONMENT_PRODUCTION = 'production';

    /** the plugin's staging production identifier */
    const ENVIRONMENT_STAGING = 'staging';

    /** @var string the credit card gateway identifier */
    const CREDIT_CARD_GATEWAY_ID = 'poynt_credit_card';

    /** @var string the pay in person gateway identifier */
    const PAYINPERSON_GATEWAY_ID = 'pay_in_person';

    /** @var Plugin single instance of this plugin */
    protected static $instance;

    /** @var array active gateways */
    protected $activeGateways;

    /** @var Support instance */
    private $support;

    /**
     * Plugin constructor.
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        $this->activeGateways = $this->getActiveGateways();

        parent::__construct(
            self::PLUGIN_ID,
            self::VERSION,
            [
                'text_domain'  => 'godaddy-payments',
                'gateways'     => $this->activeGateways,
                // TODO: Replace currencies when limitations are clear (pitch still has a TODO instruction for this one). {AC 2021-02-01}
                'currencies'   => [],
                'dependencies' => [
                    'php_extensions' => [
                        'json',
                    ],
                ],
                'supports' => [
                    self::FEATURE_CAPTURE_CHARGE,
                    self::FEATURE_MY_PAYMENT_METHODS,
                ],
                'require_ssl' => true,
            ]
        );

        $this->support = $this->getSupport();
        $this->add_hooks();
        $this->init();
    }

    /**
     * Setup hooks.
     *
     * @since x.x.x
     */
    private function add_hooks()
    {
        add_action('admin_enqueue_scripts', [$this, 'registerScripts']);
        add_action('admin_enqueue_scripts', [$this, 'registerStyles']);
    }

    /**
     * Return the activated gateways based on the conditions for Pay In Person gateway.
     *
     * @since x.x.x
     * @return array
     */
    protected function getActiveGateways(): array
    {
        $gateways = [
            self::CREDIT_CARD_GATEWAY_ID => CreditCardGateway::class,
        ];

        if (WCHelper::shouldLoadPayInPersonGateway()) {
            $gateways[self::PAYINPERSON_GATEWAY_ID] = PayInPersonGateway::class;
        }

        return $gateways;
    }

    /**
     * Adds gateway setup page stylesheet.
     *
     * @since x.x.x
     */
    public function registerScripts()
    {

        // only load the scripts on the payment methods page
        if (! WCHelper::isAccessingPaymentSettings()) {
            return;
        }

        $paymentMethods = [
            [
                'gatewayId'     => self::CREDIT_CARD_GATEWAY_ID,
                'allowEnable'   => true,
            ],
        ];

        if (array_key_exists(self::PAYINPERSON_GATEWAY_ID, $this->activeGateways)) {
            $paymentMethods[] = [
                'gatewayId'     => self::PAYINPERSON_GATEWAY_ID,
                'allowEnable'   => PoyntHelper::hasPoyntSmartTerminalActivated(),
            ];
        }

        wp_enqueue_script(self::PLUGIN_ID.'_payment_methods', $this->get_plugin_url().'/assets/js/admin/payment-methods.js', ['jquery'], self::VERSION);
        wp_localize_script(self::PLUGIN_ID.'_payment_methods', self::PLUGIN_ID.'PaymentMethods', $paymentMethods);
    }

    public function registerStyles()
    {
        wp_enqueue_style('gdp-notices', $this->get_plugin_url().'/assets/css/notices.css', [], $this->get_version());
    }

    /**
     * Initial the core functionality.
     */
    public function init()
    {
        $this->loadShippingMethods();
        $this->loadGdpEmails();
        $this->loadNotices();
        $this->loadScheduledActions();
        $this->loadWebhooksHandler();
        $this->loadPoyntOrderSynchronizer();
        $this->addCaptureSupport();
        $this->loadTransactionSynchronizer();
        $this->loadMisc();
    }

    /**
     * Gets the full path and filename of the plugin file.
     *
     * @since 1.0.0
     *
     * @return string
     */
    protected function get_file() : string
    {
        return __DIR__;
    }

    /**
     * Gets the plugin full name.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function get_plugin_name() : string
    {
        return __('GoDaddy Payments', 'godaddy-payments');
    }

    /**
     * Gets the main plugin instance.
     *
     * Ensures only one instance is/can be loaded.
     *
     * @since 1.0.0
     *
     * @return Plugin
     */
    public static function instance() : self
    {
        if (null === self::$instance) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    /**
     * Gets the plugin documentation URL.
     *
     * @NOTE this will be used by {@see CreditCardGateway::get_not_configured_error_message()} {unfulvio 2021-07-02}
     * If we want to have a different URL than the documentation URL produced there, the method needs to be overridden separately.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function get_documentation_url() : string
    {
        return 'https://www.godaddy.com/help/40417';
    }

    /**
     * Gets the reviews URL.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function get_reviews_url()
    {
        return 'https://wordpress.org/support/plugin/godaddy-payments/reviews/';
    }

    /**
     * Gets the plugin sales page URL.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function get_sales_page_url()
    {
        return 'https://payments.godaddy.com/';
    }

    /**
     * Gets the plugin support URL.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function get_support_url()
    {
        return 'https://wordpress.org/support/plugin/godaddy-payments/';
    }

    /**
     * Gets the my payment methods handler instance.
     *
     * @since 1.0.0
     *
     * @return PaymentMethods
     */
    protected function get_my_payment_methods_instance() : PaymentMethods
    {
        return new PaymentMethods($this);
    }

    /**
     * Loads and initializes the plugin lifecycle handler.
     *
     * @since 1.0.0
     */
    protected function init_lifecycle_handler()
    {
        $this->lifecycle_handler = new Lifecycle($this);
    }

    /**
     * Loads and initializes the plugin REST API handler.
     *
     * @since 1.2.0
     */
    protected function init_rest_api_handler()
    {
        require_once $this->get_payment_gateway_framework_path().'/rest-api/class-sv-wc-payment-gateway-plugin-rest-api.php';

        $this->rest_api_handler = new REST($this);
    }

    /**
     * Gets the support handler instance.
     *
     * @since 1.2.0
     *
     * @return Support
     */
    public function getSupport() : Support
    {
        return $this->support instanceof Support ? $this->support : new Support();
    }

    /**
     * Adds any admin notices upon initialization.
     *
     * @internal
     *
     * @since 1.0.0
     */
    public function add_admin_notices()
    {
        parent::add_admin_notices();

        if ($this->get_lifecycle_handler()->hasMigratedFromLegacyPlugin()) {
            $this->get_admin_notice_handler()->add_admin_notice(
                sprintf(
                    /* translators: Placeholders: %1$s - opening <a> HTML link tag, %2$s - closing </a> HTML link tag */
                    __('Migration successful! WooCommerce Poynt Collect Payment Gateway was deactivated, and Poynt &mdash; a GoDaddy Brand for WooCommerce has been %1$sconfigured with your previous settings%2$s.', 'godaddy-payments'),
                    '<a href="'.esc_url($this->get_settings_url()).'">', '</a>'
                ),
                Lifecycle::MIGRATED_FROM_LEGACY_PLUGIN_FLAG,
                [
                    'dismissible'             => true,
                    'always_show_on_settings' => false,
                    'notice_class'            => 'updated',
                ]
            );
        }
    }

    /**
     * Adds the delayed admin notices.
     *
     * @internal
     *
     * @since 1.0.0
     */
    public function add_delayed_admin_notices()
    {
        parent::add_delayed_admin_notices();

        $this->add_new_user_notices();
    }

    /**
     * Adds notices to prompt new user configuration.
     *
     * @since 1.0.0
     */
    private function add_new_user_notices()
    {
        /** @var CreditCardGateway $settings_gateway */
        $settings_gateway = $this->get_gateway(self::CREDIT_CARD_GATEWAY_ID);

        // warn and bail if the shop's currency isn't supported
        if ($this->maybeAddCurrencyAdminNotice()) {
            return;
        }

        // warn and bail if the shop's base isn't US
        if ($this->maybeAddCountryAdminNotice()) {
            return;
        }

        // show the following notices conditionally depending whether the admin is or isn't on the gateway settings page
        if (! $settings_gateway || ! $this->is_payment_gateway_configuration_page($settings_gateway->get_id())) {
            $this->maybeAddOnboardingNotice();

            return;
        }

        // show the signup prompt if the production credentials are empty
        $this->maybeAddSignupAdminNotice($settings_gateway);
    }

    /**
     * Gets the signup URL.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function getSignupUrl() : string
    {
        return 'https://signup.payments.godaddy.com/r/woo-plugin';
    }

    /**
     * Determines whether the current store country is supported or not.
     *
     * @since 1.1.0
     *
     * @return bool
     */
    public function isCountrySupported() : bool
    {
        return WC()->countries && 'US' === WC()->countries->get_base_country();
    }

    /**
     * Determines whether the current store currency is supported or not.
     *
     * @since 1.1.0
     *
     * @return bool
     */
    public function isCurrencySupported() : bool
    {
        return 'USD' === get_woocommerce_currency();
    }

    /**
     * May add an admin notice if the store's currency is not supported.
     *
     * @since 1.1.0
     *
     * @return bool true if the notice was added
     */
    private function maybeAddCurrencyAdminNotice() : bool
    {
        if ($this->isCurrencySupported()) {
            return false;
        }

        $this->get_admin_notice_handler()->add_admin_notice(
            sprintf(
                /* translators: Placeholders: %1$s - opening <a> HTML link tag, %2$s - closing </a> HTML link tag */
                __('%1$s requires U.S. dollar transactions. Please %2$schange your Currency%3$s in order to use %1$s.', 'godaddy-payments'),
                    $this->get_plugin_name(),
                    '<a href="'.esc_url($this->get_general_configuration_url()).'">',
                    '</a>'
            ),
            'invalid-currency-prompt',
            [
                'notice_class' => 'notice-error',
            ]
        );

        return true;
    }

    /**
     * May add an admin notice if the store's country is not supported.
     *
     * @since 1.1.0
     *
     * @return bool true if the notice was added
     */
    private function maybeAddCountryAdminNotice() : bool
    {
        if ($this->isCountrySupported()) {
            return false;
        }

        $this->get_admin_notice_handler()->add_admin_notice(
            sprintf(
                /* translators: Placeholders: %1$s - opening <a> HTML link tag, %2$s - closing </a> HTML link tag */
                __('%1$s is available for United States-based businesses. Please %2$supdate your Store Address%3$s if you are located in the U.S.', 'godaddy-payments'),
                $this->get_plugin_name(),
                '<a href="'.esc_url($this->get_general_configuration_url()).'">',
                '</a>'
            ),
            'non-us-prompt',
            [
                'notice_class' => 'notice-error',
            ]
        );

        return true;
    }

    /**
     * May add an admin notice to prompt merchant sign up.
     *
     * @since 1.1.0
     *
     * @param CreditCardGateway $settings_gateway the gateway settings
     */
    private function maybeAddSignupAdminNotice(SV_WC_Payment_Gateway $settings_gateway)
    {
        if (! $settings_gateway->getAppId(self::ENVIRONMENT_PRODUCTION) || ! $settings_gateway->getPrivateKey(self::ENVIRONMENT_PRODUCTION)) {
            $this->get_admin_notice_handler()->add_admin_notice(
                sprintf(
                    /* translators: Placeholders: %1$s - Plugin name, %2$s - opening <a> HTML link tag, %3$s - closing </a> HTML link tag */
                    __('Start taking secure payments in minutes and get paid faster with deposits as soon as the next day. Accept any major credit/debit
                    card directly on your US-based WooCommerce site for just %1$s + 30&cent; per online transaction. %2$sSign Up Now%3$s', 'godaddy-payments'),
                    '2.3%',
                    '<strong><a href="'.esc_url($this->getSignupUrl()).'" target="_blank">', '</a></strong>'
                ),
                'signup-prompt',
                [
                    'dismissible'             => true,
                    'always_show_on_settings' => false,
                    'notice_class'            => 'notice-info',
                ]
            );
        }
    }

    /**
     * May add an admin notice if the payment gateway is not enabled.
     *
     * Checks if the credentials are empty too, assuming this is when the plugin is first activated and never configured before.
     *
     * @since 1.1.3
     */
    private function maybeAddOnboardingNotice()
    {
        $settings_gateway = $this->get_gateway(self::CREDIT_CARD_GATEWAY_ID);

        if ($settings_gateway && ! $settings_gateway->is_configured() && ! $settings_gateway->is_enabled()) {
            $this->get_admin_notice_handler()->add_admin_notice(
                sprintf(
                    /* translators: Placeholders: %1$s - Plugin name, %2$s - opening <a> HTML link tag, %3$s - closing </a> HTML link tag */
                    __('Set up %1$s for WooCommerce to securely accept credit and debit card payments in your checkout, and keep more of your money with the industry\'s lowest fees - just %2$s + 30&cent; per online transaction. %3$sSet up now%4$s', 'godaddy-payments'),
                        $this->get_plugin_name(),
                        '2.3%',
                        '<strong><a href="'.esc_url($this->get_settings_url($settings_gateway->get_id())).'">',
                        '</a></strong>'

                ),
                'poynt-disabled-no-credentials',
                [
                    'dismissible'             => true,
                    'always_show_on_settings' => false,
                    'notice_class'            => 'notice-info',
                ]
            );
        }
    }

    /**
     * Load the scheduled action methods.
     *
     * @since 1.3.0
     */
    protected function loadScheduledActions()
    {
        new ActiveSmartTerminalDetector();
    }

    /**
     * Load the shipping methods.
     *
     * @since 1.3.0
     */
    protected function loadShippingMethods()
    {
        new CoreShippingMethods();
        new LocalPickup();
    }

    /**
     * load woo emails.
     *
     * @since 1.3.0
     */
    protected function loadGdpEmails()
    {
        new GDPEmails();
    }

    /**
     * load Notices.
     *
     * 1.3.0
     */
    protected function loadNotices()
    {
        new Notices();
    }

    /**
     * load webhooks handlers.
     *
     * @since 1.3.0
     */
    protected function loadWebhooksHandler()
    {
        new PoyntWebhooksHandler();
    }

    /**
     * load Sync Poynt Order class.
     *
     * @since 1.3.0
     */
    protected function loadPoyntOrderSynchronizer()
    {
        new PoyntOrderSynchronizer();
    }

    protected function addCaptureSupport()
    {
        new Captures();
    }

    /**
     * load Sync Transactions class.
     *
     * @since 1.3.0
     */
    protected function loadTransactionSynchronizer()
    {
        new PoyntTransactionSynchronizer();
    }

    /**
     * load Miscellaneous things.
     *
     * @since 1.3.0
     */
    protected function loadMisc()
    {
        new ViewOrderPage();
    }
}
